#! /usr/bin/python

'''
1. display ovsdb log
ovsdb-tool show-log -m /ovs/ovs-vswitchd.conf.db

2. list core files
ls /pica/core

3. display error message in the /tmp/log/messages
grep -i err /tmp/log/messages
grep -i fail /tmp/log/messages

4. soft flow table info
software:
ovs-ofctl dump-flows br0

5. hard flow table info
hardware:
ovs-appctl pica/dump-flows

6. display system version
version

7. all Open_vSwitch table
ovsdb-client dump Open_vSwitch

8. display port status
ovs-ofctl show br0

9. proces status
ps aux
'''

import subprocess
import re
import os
import datetime

# get the output of the specific command
def get_cmd_output(command=None):

    # call subprocess to excute the command
    p = subprocess.Popen(command, stdout=subprocess.PIPE, bufsize=-1, shell=True, close_fds=True)
    (output, err) = p.communicate()
    p_status = p.wait()

    if p.stdin:
        p.stdin.close()
    if p.stdout:
        p.stdout.close()
    if p.stderr:
        p.stderr.close()

    return output

# get the bridge name
def get_bridge_name():

    # get the bridge
    output = get_cmd_output(command='ovs-vsctl show | grep Bridge')
    bridges = re.findall('(?i)Bridge "?(.*[^\n\r"])"?', output)

    return bridges

# excute the show_tech_support
def show_tech_support():

    print 'Start......%s' % os.linesep

    now = datetime.datetime.now()
    other = now.strftime("%Y%m%d%H%M%S")

    fobj = open('/tmp/Ovs-%s-techSupport.log' % other, 'w')

    fobj.write('Item 1: Display ovsdb log%s' % os.linesep)
    if os.path.exists('/ovs/ovs-vswitchd.conf.db'):
        ovsdb_log = get_cmd_output(command='ovsdb-tool show-log -m /ovs/ovs-vswitchd.conf.db')
        fobj.write('%s%s' % (ovsdb_log, os.linesep))
    print 'Item 1: Display ovsdb log finished!'

    fobj.write('Item 2: Display ovs core files%s' % os.linesep)
    core_log = get_cmd_output(command='ls /pica/core')
    fobj.write('%s%s' % (core_log, os.linesep))
    print 'Item 2: Display ovs core files finished!'

    fobj.write('Item 3: Display err log%s' % os.linesep)
    if os.path.exists('/tmp/log/messages'):
        err_log = get_cmd_output(command='grep -iE "err|fail" /tmp/log/messages')
        fobj.write('%s%s' % (err_log, os.linesep))
    print 'Item 3: Display err log finished!'

    fobj.write('Item 4: Display software flow table%s' % os.linesep)
    if os.path.exists('/ovs/var/run/openvswitch/db.sock'):
        bridges = get_bridge_name()
        for bridge in bridges:
            soft_flow_log = get_cmd_output(command='ovs-ofctl dump-flows %s' % bridge)
            fobj.write('%s%s' % (soft_flow_log, os.linesep))
    print 'Item 4: Display software flow table finished!'

    fobj.write('Item 5: Display hardware flow table%s' % os.linesep)
    if os.path.exists('/ovs/var/run/openvswitch/db.sock'):
        hard_flow_log = get_cmd_output(command='ovs-appctl pica/dump-flows')
        fobj.write('%s%s' % (hard_flow_log, os.linesep))
    print 'Item 5: Display hardware flow table finished!'

    fobj.write('Item 6: Display system version%s' % os.linesep)
    version_log = get_cmd_output(command='version')
    fobj.write('%s%s' % (version_log, os.linesep))
    print 'Item 6: Display system version finished!'

    fobj.write('Item 7: Display Open_vSwitch table%s' % os.linesep)
    if os.path.exists('/ovs/var/run/openvswitch/db.sock'):
        ovs_log = get_cmd_output(command='ovsdb-client dump Open_vSwitch ')
        fobj.write('%s%s' % (ovs_log, os.linesep))
    print 'Item 7: Display Open_vSwitch table finished!'

    fobj.write('Item 8: Display port status%s' % os.linesep)
    if os.path.exists('/ovs/var/run/openvswitch/db.sock'):
        for bridge in bridges:
            port_log = get_cmd_output(command='ovs-ofctl show %s' % bridge)
            fobj.write('%s%s' % (port_log, os.linesep))
    print 'Item 8: Display port status finished!'

    fobj.write('Item 9: Display process%s' % os.linesep)
    process_log = get_cmd_output(command='ps aux')
    fobj.write('%s%s' % (process_log, os.linesep))
    print 'Item 9: Display process finished!'

    fobj.write('Item 10: Display license%s' % os.linesep)
    license_log = get_cmd_output(command='license -s')
    fobj.write('%s%s' % (license_log, os.linesep))
    print 'Item 10: Display license finished!'

    fobj.close()

    print '%sThe information has been stored in /tmp/Ovs-%s-techSupport.log, please forward to support@pica8.com%s' % (os.linesep, other, os.linesep)

if __name__ == '__main__':
    show_tech_support()
